
/*
 * $Id: Server.cc,v 1.1.1.5 1999/02/22 06:19:48 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include <strstream.h>

#include "Connection.h"
#include "Server.h"
#include "SrvConnMgr.h"
#include "ListenQueue.h"
#include "LogComment.h"
#include "StatPhaseMgr.h"
#include "Rnd.h"
#include "SharedOpts.h"
#include "SrvOpts.h"

#include "polyErrors.h"
#include "gadgets.h"
#include "globals.h"



Server::Server(): theLQueue(0), theThinkDistr(0), thePort(-1) {
	theConnMgr = new SrvConnMgr; // for now
}

Server::~Server() {
	delete theConnMgr;
	deaf();
}

void Server::configure(int aPort, RndDistr *aThinkDistr) {
	thePort = aPort;
	theThinkDistr = aThinkDistr;
	theXacts.limit(1024);

	if (!theSock.create())
		FatalError(errno);
	theSock.blocking(false);
	theSock.reuseAddr(true);
	if (TheOpts.theLingerTimeout >= 0)
		theSock.linger(TheOpts.theLingerTimeout.sec());
	if (!TheOpts.doUseNagle)
		theSock.tcpDelay(false);
	if (!theSock.listen(thePort))
		FatalError(errno);
}

void Server::listenQueue(ListenQueue *aQueue) {
	Assert(aQueue);
	Assert(theSock);
	theLQueue = aQueue;
	theLQueue->listener(this, theSock.fd());
}

void Server::start() {
	Assert(theSock);
	Assert(theLQueue);
	theReserv = TheSelect->setFD(theSock.fd(), dirRead, this);
}

void Server::noteXactDone(SrvXact *x) {
	Assert(x);
	theConnMgr->put(x->conn());
	theXacts.put(x);
	theLQueue->noteClose(); // XXX: wrong place for pconns
}

void Server::noteReadReady(int) {
	int count = 0;
	// note: negative theAcceptLmt means no limit
	for (int i = TheSrvOpts.theAcceptLmt; i; --i) {
		bool fatal;
		if (Connection *conn = theConnMgr->get(theSock, fatal)) {
			startXact(conn);
			count++;
			continue;
		}

		if (fatal) {
			deaf();
			FatalError(errServerGone);
		}
		break;
	}
	theLQueue->accepted(count);
}

void Server::noteWriteReady(int) {
	Assert(0);
}

void Server::deaf() {
	if (theSock) {
		Comment << "server closing listen socket " << theSock.fd() << " on port " << thePort << endc;
		theSock.close();
	}
	if (theReserv && TheSelect)
		TheSelect->clearFD(theReserv);
	theLQueue->listener(0, -1);
}
		
void Server::startXact(Connection *conn) {
	SrvXact *x = theXacts.get();
	x->exec(this, conn,
		theThinkDistr ? Time::Secd(theThinkDistr->trial()) : Time());
}
