
/*
 * $Id: ILog.h,v 1.1.1.3 1999/03/08 07:56:10 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef I_LOG_CLASS_H
#define I_LOG_CLASS_H

#include <iostream.h>

#include "Time.h"
#include "Size.h"
#include "NetDouble.h"
#include "String.h"
#include "Array.h"
#include "LogMeta.h"
#include "Progress.h"

// binary input log
class ILog {
	protected:
		typedef void (Manip)(class ILog &);

	public:
		ILog();
		~ILog();

		void stream(const String &aFileName, istream *aStream);
		istream *stream() { return theStream; }
		operator void*() { return theStream && theStream->good() ? theStream : 0; }

		ILog &operator >>(int &x) { geti(x); return *this; }
		ILog &operator >>(double &x) { getd(x); return *this; }
		ILog &operator >>(Size &sz) { sz = geti(); return *this; }
		ILog &operator >>(String &s) { gets(s); return *this; }
		ILog &operator >>(Time &t) { t.tv_sec = geti(); t.tv_usec = geti(); return *this; }

		ILog &operator >>(Manip m) { m(*this); return *this; }

		int geti() { int x; return(geti(x)); }
		int geti(int &x) { get(&x, sizeof(x)); return x = ntohl(x); }
		int geti(int *&xs, int &count); // returns count
		double getd(double &x) { NetDouble nd; geti(nd.mnt); geti(nd.exp); return x = nd; }
		String &gets(String &s);

		LogEntryPrefix begEntry();
		void endEntry();

		void loadHeader();
		const LogTagDescr *tagDescr(int tag) const { return (tag > 0 && tag <= theTagMax) ? theSymbTable[tag] : 0; }
		const LogTagDescr *tagDescr(const String &name) const;

		const ProgrRec &progress() const { return theProgress; }

	protected:
		void get(void *buf, int len) { theStream->read(buf, len); }
		void loadSymbTable();

	protected:
		istream *theStream;
		String theFileName;
		Array<LogTagDescr*> theSymbTable;
		int theTagMax;       // maximal tag in symbol table

		streampos theCurPos; // start of a current entry
		LogEntryPrefix theCurPx; // prefix of the current entry
		ProgrRec theProgress;  // current "local time", #errs, etc.
};


template <class Item>
inline
ILog &operator >>(ILog &log, Array<Item> &a) {
	const int cnt = log.geti();
	if (a.count()) { // use existing entries
		Assert(cnt <= a.count());
		for (int i = 0; i < cnt; ++i)
			log >> a.item(i);
	} else {
		for (int i = 0; i < cnt; ++i) {
			Item item;
			log >> item;
			a << item;
		}
	}
	return log;
}

#endif
