
/*
 * $Id: Progress.h,v 1.1.2.3 1999/03/08 07:56:12 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef PROGRESS_CLASS_H
#define PROGRESS_CLASS_H

#include "Clock.h"

class OLog;
class ILog;
class ostream;

// errors can be measured in absolute and relative quantaties
class ErrRec {
	public:
		ErrRec();

		bool enough(int xc, int ec) const { return enoughC(ec) || enoughR(ec, xc+ec); }

		ostream &print(ostream &os) const;

	protected:
		bool enoughC(int ec) const { return theCnt >= 0 && ec >= theCnt; }
		bool enoughR(int ec, int xec) const { return theRatio > 0 && xec >= 1000 && ec >= xec*theRatio; }

	public:
		int theCnt;
		double theRatio; // #errs/#xact
};

// maintains information about simulation progress (or simulation goal)
class ProgrRec {
	public:
		ProgrRec();

		void store(OLog &) const;
		void load(ILog &);
		ostream &print(ostream &os) const;

	public:
		Time theDuration;
		int theXactComplCnt;
		ErrRec theErrs;
};

// automagically updates duration; ctor set counters to zero
class Progress: public ProgrRec, public Clock {
	public:
		Progress();

		virtual void update(Time aCurTime);

	protected:
		Time theStartTime;
};

// progress target
class Goal: public ProgrRec {
	public:
		bool enoughErrs(const ProgrRec &prg) const { return theErrs.enough(prg.theXactComplCnt, prg.theErrs.theCnt); }
		operator void *() const; // is set?
		Goal &operator /=(double factor);

		Goal &operator +=(const Goal &p);

};



/* inline operators */

inline ostream &operator <<(ostream &os, const ProgrRec &r) { return r.print(os); }
inline OLog &operator <<(OLog &ol, const ProgrRec &r) { r.store(ol); return ol; }
inline ILog &operator >>(ILog &il, ProgrRec &r) { r.load(il); return il; }

inline 
bool operator >=(const ProgrRec &cur, const Goal &goal) {
	return // reaching one sub-goal is enough!
		(cur.theXactComplCnt >= goal.theXactComplCnt && goal.theXactComplCnt >= 0) ||
		goal.enoughErrs(cur) ||
		(cur.theDuration > goal.theDuration && goal.theDuration >= 0)
		;
}
inline 
bool operator <(const ProgrRec &cur, const Goal &goal) {
	return !(cur >= goal);
}

inline 
Goal operator /(const Goal &g, double factor) { return Goal(g) /= factor; }

#endif
